/* 
 * Copyright (c) 2003 Jean-Yves Lefort <jylefort@brutele.be>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "config.h"
#include <gnome.h>
#include <string.h>
#include "mn-conf.h"
#include "mn-dialog.h"
#include "glade-interface.h"
#include "glade-support.h"
#include "mn-mailbox.h"
#include "mn-mailboxes.h"
#include "mn-preferences.h"
#include "mn-util.h"

/*** types *******************************************************************/

enum {
  COLUMN_PTR,
  COLUMN_MAILBOX,
  COLUMN_FORMAT,
  N_COLUMNS
};

/*** variables ***************************************************************/

static struct
{
  GtkWidget	*dialog;

  GtkWidget	*local_enabled;
  GtkWidget	*local_minutes_spin;
  GtkWidget	*local_minutes_label;
  GtkWidget	*local_seconds_spin;
  GtkWidget	*local_seconds_label;

  GtkWidget	*remote_enabled;
  GtkWidget	*remote_minutes_spin;
  GtkWidget	*remote_minutes_label;
  GtkWidget	*remote_seconds_spin;
  GtkWidget	*remote_seconds_label;

  GtkWidget	*list;
  GtkWidget	*remove;

  GtkWidget	*command_new_mail;
  GtkWidget	*command_new_mail_entry;
  GtkWidget	*command_clicked;
  GtkWidget	*command_clicked_entry;

  GtkListStore	*store;
} mn_preferences = { NULL, };

/*** functions ***************************************************************/

static void mn_preferences_add_column		(int              id,
						 const char       *label);
static void mn_preferences_bind_boolean		(GtkWidget        *widget,
						 const char       *key);
static void mn_preferences_boolean_toggled	(GtkToggleButton  *button,
						 gpointer         data);
static void mn_preferences_bind_int		(GtkWidget        *widget,
						 const char       *key);
static void mn_preferences_int_changed		(GtkSpinButton    *button,
						 gpointer         data);
static void mn_preferences_bind_string		(GtkWidget        *widget,
						 const char       *key);
static void mn_preferences_string_changed	(GtkEditable      *editable,
						 gpointer         data);
static void mn_preferences_selection_changed	(GtkTreeSelection *selection,
						 gpointer         data);
static void mn_preferences_remove_mailbox_cb	(GtkTreeModel     *model,
						 GtkTreePath      *path,
						 GtkTreeIter      *iter,
						 gpointer         data);

/*** implementation **********************************************************/

static void
mn_preferences_add_column (int id, const char *label)
{
  GtkTreeViewColumn *column;

  g_return_if_fail(label != NULL);

  column = gtk_tree_view_column_new_with_attributes(label,
						    gtk_cell_renderer_text_new(),
						    "text", id,
						    NULL);
  gtk_tree_view_column_set_sort_column_id(column, id);
  gtk_tree_view_column_set_resizable(column, TRUE);
  gtk_tree_view_append_column(GTK_TREE_VIEW(mn_preferences.list), column);
}

void
mn_preferences_display (void)
{
  GtkSizeGroup *size_group;
  GtkTreeSelection *selection;

  if (mn_preferences.dialog)
    {
      gtk_window_present(GTK_WINDOW(mn_preferences.dialog));
      return;
    }
  
  mn_preferences.dialog = create_preferences();
  mn_preferences.local_enabled = lookup_widget(mn_preferences.dialog, "preferences_local_enabled");
  mn_preferences.local_minutes_spin = lookup_widget(mn_preferences.dialog, "preferences_local_minutes_spin");
  mn_preferences.local_minutes_label = lookup_widget(mn_preferences.dialog, "preferences_local_minutes_label");
  mn_preferences.local_seconds_spin = lookup_widget(mn_preferences.dialog, "preferences_local_seconds_spin");
  mn_preferences.local_seconds_label = lookup_widget(mn_preferences.dialog, "preferences_local_seconds_label");
  mn_preferences.remote_enabled = lookup_widget(mn_preferences.dialog, "preferences_remote_enabled");
  mn_preferences.remote_minutes_spin = lookup_widget(mn_preferences.dialog, "preferences_remote_minutes_spin");
  mn_preferences.remote_minutes_label = lookup_widget(mn_preferences.dialog, "preferences_remote_minutes_label");
  mn_preferences.remote_seconds_spin = lookup_widget(mn_preferences.dialog, "preferences_remote_seconds_spin");
  mn_preferences.remote_seconds_label = lookup_widget(mn_preferences.dialog, "preferences_remote_seconds_label");
  mn_preferences.list = lookup_widget(mn_preferences.dialog, "preferences_list");
  mn_preferences.remove = lookup_widget(mn_preferences.dialog, "preferences_remove");
  mn_preferences.command_new_mail = lookup_widget(mn_preferences.dialog, "preferences_command_new_mail");
  mn_preferences.command_new_mail_entry = lookup_widget(mn_preferences.dialog, "preferences_command_new_mail_entry");
  mn_preferences.command_clicked = lookup_widget(mn_preferences.dialog, "preferences_command_clicked");
  mn_preferences.command_clicked_entry = lookup_widget(mn_preferences.dialog, "preferences_command_clicked_entry");

  /* finish the mailboxes list */

  mn_preferences.store = gtk_list_store_new(N_COLUMNS,
					    MN_TYPE_MAILBOX,
					    G_TYPE_STRING,
					    G_TYPE_STRING);
  gtk_tree_view_set_model(GTK_TREE_VIEW(mn_preferences.list), GTK_TREE_MODEL(mn_preferences.store));

  mn_preferences_add_column(COLUMN_MAILBOX, _("Mailbox"));
  mn_preferences_add_column(COLUMN_FORMAT, _("Format"));

  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(mn_preferences.list));
  gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);
  g_signal_connect(G_OBJECT(selection), "changed",
		   G_CALLBACK(mn_preferences_selection_changed), NULL);
  
  /* create the GtkSizeGroup */
  
  size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
  gtk_size_group_add_widget(size_group, mn_preferences.local_enabled);
  gtk_size_group_add_widget(size_group, mn_preferences.remote_enabled);
  gtk_size_group_add_widget(size_group, mn_preferences.command_new_mail);
  gtk_size_group_add_widget(size_group, mn_preferences.command_clicked);

  mn_preferences_update_values();
  mn_preferences_update_sensitivity();

  mn_preferences_bind_boolean(mn_preferences.local_enabled, "/apps/mail-notification/local/enabled");
  mn_preferences_bind_int(mn_preferences.local_minutes_spin, "/apps/mail-notification/local/delay/minutes");
  mn_preferences_bind_int(mn_preferences.local_seconds_spin, "/apps/mail-notification/local/delay/seconds");
  mn_preferences_bind_boolean(mn_preferences.remote_enabled, "/apps/mail-notification/remote/enabled");
  mn_preferences_bind_int(mn_preferences.remote_minutes_spin, "/apps/mail-notification/remote/delay/minutes");
  mn_preferences_bind_int(mn_preferences.remote_seconds_spin, "/apps/mail-notification/remote/delay/seconds");
  mn_preferences_bind_boolean(mn_preferences.command_new_mail, "/apps/mail-notification/commands/new-mail/enabled");
  mn_preferences_bind_string(mn_preferences.command_new_mail_entry, "/apps/mail-notification/commands/new-mail/command");
  mn_preferences_bind_boolean(mn_preferences.command_clicked, "/apps/mail-notification/commands/clicked/enabled");
  mn_preferences_bind_string(mn_preferences.command_clicked_entry, "/apps/mail-notification/commands/clicked/command");

  mn_dialog_show(GTK_DIALOG(mn_preferences.dialog), &mn_preferences.dialog);
}

void
mn_preferences_update_values (void)
{
  if (mn_preferences.dialog)
    {
      const char *command_new_mail;
      const char *command_clicked;
      GSList *iter;
      
      command_new_mail = mn_conf_get_string("/apps/mail-notification/commands/new-mail/command");
      command_clicked = mn_conf_get_string("/apps/mail-notification/commands/clicked/command");
      
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(mn_preferences.local_enabled),
				   mn_conf_get_bool("/apps/mail-notification/local/enabled"));
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(mn_preferences.local_minutes_spin),
				mn_conf_get_int("/apps/mail-notification/local/delay/minutes"));
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(mn_preferences.local_seconds_spin),
				mn_conf_get_int("/apps/mail-notification/local/delay/seconds"));
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(mn_preferences.remote_enabled),
				   mn_conf_get_bool("/apps/mail-notification/remote/enabled"));
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(mn_preferences.remote_minutes_spin),
				mn_conf_get_int("/apps/mail-notification/remote/delay/minutes"));
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(mn_preferences.remote_seconds_spin),
				mn_conf_get_int("/apps/mail-notification/remote/delay/seconds"));
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(mn_preferences.command_new_mail),
				   mn_conf_get_bool("/apps/mail-notification/commands/new-mail/enabled"));
      gtk_entry_set_text(GTK_ENTRY(mn_preferences.command_new_mail_entry), command_new_mail ? command_new_mail : "");
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(mn_preferences.command_clicked),
				   mn_conf_get_bool("/apps/mail-notification/commands/clicked/enabled"));
      gtk_entry_set_text(GTK_ENTRY(mn_preferences.command_clicked_entry), command_clicked ? command_clicked : "");

      gtk_list_store_clear(mn_preferences.store);
      MN_UTIL_SLIST_FOREACH(iter, mn_mailboxes)
	{
	  MNMailbox *mailbox = (MNMailbox *) iter->data;
	  MNMailboxClass *class;
	  GtkTreeIter tree_iter;
	  
	  class = g_type_class_peek(G_TYPE_FROM_INSTANCE(mailbox));

	  gtk_list_store_append(mn_preferences.store, &tree_iter);
	  gtk_list_store_set(mn_preferences.store,
			     &tree_iter,
			     COLUMN_PTR, mailbox,
			     COLUMN_MAILBOX, mailbox->name,
			     COLUMN_FORMAT, class->format,
			     -1);
	}
    }
}

void
mn_preferences_update_sensitivity (void)
{
  gboolean local_check_enabled;
  gboolean remote_check_enabled;
  gboolean command_new_mail_enabled;
  gboolean command_clicked_enabled;
  GtkTreeSelection *selection;
  gboolean has_selection;

  local_check_enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(mn_preferences.local_enabled));

  gtk_widget_set_sensitive(mn_preferences.local_minutes_spin, local_check_enabled);
  gtk_widget_set_sensitive(mn_preferences.local_minutes_label, local_check_enabled);
  gtk_widget_set_sensitive(mn_preferences.local_seconds_spin, local_check_enabled);
  gtk_widget_set_sensitive(mn_preferences.local_seconds_label, local_check_enabled);

  remote_check_enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(mn_preferences.remote_enabled));

  gtk_widget_set_sensitive(mn_preferences.remote_minutes_spin, remote_check_enabled);
  gtk_widget_set_sensitive(mn_preferences.remote_minutes_label, remote_check_enabled);
  gtk_widget_set_sensitive(mn_preferences.remote_seconds_spin, remote_check_enabled);
  gtk_widget_set_sensitive(mn_preferences.remote_seconds_label, remote_check_enabled);

  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(mn_preferences.list));
  has_selection = gtk_tree_selection_count_selected_rows(selection) > 0;

  gtk_widget_set_sensitive(mn_preferences.remove, has_selection);

  command_new_mail_enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(mn_preferences.command_new_mail));
  command_clicked_enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(mn_preferences.command_clicked));

  gtk_widget_set_sensitive(mn_preferences.command_new_mail_entry, command_new_mail_enabled);
  gtk_widget_set_sensitive(mn_preferences.command_clicked_entry, command_clicked_enabled);
}

static void
mn_preferences_bind_boolean (GtkWidget *widget, const char *key)
{
  g_signal_connect(G_OBJECT(widget),
		   "toggled",
		   G_CALLBACK(mn_preferences_boolean_toggled),
		   key);
}

static void
mn_preferences_boolean_toggled (GtkToggleButton *button,
			     gpointer data)
{
  const char *key = (const char *) data;
  mn_conf_set_bool(key, gtk_toggle_button_get_active(button));
}

static void
mn_preferences_bind_int (GtkWidget *widget, const char *key)
{
  g_signal_connect(G_OBJECT(widget),
		   "value-changed",
		   G_CALLBACK(mn_preferences_int_changed),
		   key);
}

static void
mn_preferences_int_changed (GtkSpinButton *button,
			 gpointer data)
{
  const char *key = (const char *) data;
  mn_conf_set_int(key, gtk_spin_button_get_value(button));
}

static void
mn_preferences_bind_string (GtkWidget *widget, const char *key)
{
  g_signal_connect(G_OBJECT(widget),
		   "changed",
		   G_CALLBACK(mn_preferences_string_changed),
		   key);
}

static void
mn_preferences_string_changed (GtkEditable *editable, gpointer data)
{
  const char *key = (const char *) data;
  const char *str;

  str = gtk_editable_get_chars(editable, 0, -1);
  if (*str)
    mn_conf_set_string(key, gtk_editable_get_chars(editable, 0, -1));
  else
    mn_conf_unset(key);
}

static void
mn_preferences_selection_changed (GtkTreeSelection *selection, gpointer data)
{
  mn_preferences_update_sensitivity();
}

void
mn_preferences_add_local_mailbox (void)
{
  GtkWidget *file_selection;
  static char *path = NULL;
  
  file_selection = create_file_selection();

  if (path)
    gtk_file_selection_set_filename(GTK_FILE_SELECTION(file_selection), path);
  gtk_file_selection_set_select_multiple(GTK_FILE_SELECTION(file_selection), TRUE);

  if (gtk_dialog_run(GTK_DIALOG(file_selection)) == GTK_RESPONSE_OK)
    {
      char **selections;
      GSList *gconf_mailboxes;
      int i;
      char *tmp_path;

      /* remember path for next time */
      tmp_path = g_path_get_dirname(gtk_file_selection_get_filename(GTK_FILE_SELECTION(file_selection)));
      g_free(path);
      path = g_strconcat(tmp_path, "/", NULL);
      g_free(tmp_path);

      selections = gtk_file_selection_get_selections(GTK_FILE_SELECTION(file_selection));
      gconf_mailboxes = mn_conf_get_list("/apps/mail-notification/mailboxes",
					 GCONF_VALUE_STRING);

      for (i = 0; selections[i]; i++)
	{
	  char *mailbox;

	  mailbox = g_filename_to_utf8(selections[i], -1, NULL, NULL, NULL);
	  gconf_mailboxes = g_slist_append(gconf_mailboxes, mailbox);
	}

      g_strfreev(selections);
      
      mn_conf_set_list("/apps/mail-notification/mailboxes",
		       GCONF_VALUE_STRING,
		       gconf_mailboxes);
      mn_util_slist_free(gconf_mailboxes);
    }
  
  gtk_widget_destroy(file_selection);
}

void
mn_preferences_add_remote_mailbox (void)
{
  GtkWidget *dialog;
  GtkSizeGroup *size_group;
  GtkWidget *hostname_label;
  GtkWidget *port_label;
  GtkWidget *username_label;
  GtkWidget *password_label;
  
  dialog = create_add_pop3();

  hostname_label = lookup_widget(dialog, "add_pop3_hostname_label");
  port_label = lookup_widget(dialog, "add_pop3_port_label");
  username_label = lookup_widget(dialog, "add_pop3_username_label");
  password_label = lookup_widget(dialog, "add_pop3_password_label");

  size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
  gtk_size_group_add_widget(size_group, hostname_label);
  gtk_size_group_add_widget(size_group, port_label);
  gtk_size_group_add_widget(size_group, username_label);
  gtk_size_group_add_widget(size_group, password_label);

  if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK)
    {
      GtkWidget *hostname_entry;
      GtkWidget *port_entry;
      GtkWidget *username_entry;
      GtkWidget *password_entry;
      char *locator;
      GSList *gconf_mailboxes;

      hostname_entry = lookup_widget(dialog, "add_pop3_hostname_entry");
      port_entry = lookup_widget(dialog, "add_pop3_port_entry");
      username_entry = lookup_widget(dialog, "add_pop3_username_entry");
      password_entry = lookup_widget(dialog, "add_pop3_password_entry");

      locator = g_strdup_printf("pop3:%s:%s@%s:%s",
				gtk_editable_get_chars(GTK_EDITABLE(username_entry), 0, -1),
				gtk_editable_get_chars(GTK_EDITABLE(password_entry), 0, -1),
				gtk_editable_get_chars(GTK_EDITABLE(hostname_entry), 0, -1),
				gtk_editable_get_chars(GTK_EDITABLE(port_entry), 0, -1));

      gconf_mailboxes = mn_conf_get_list("/apps/mail-notification/mailboxes",
					 GCONF_VALUE_STRING);
      gconf_mailboxes = g_slist_append(gconf_mailboxes, locator);
      mn_conf_set_list("/apps/mail-notification/mailboxes",
		       GCONF_VALUE_STRING,
		       gconf_mailboxes);
      mn_util_slist_free(gconf_mailboxes); /* this frees locator aswell */
    }
  
  gtk_widget_destroy(dialog);
}

void
mn_preferences_remove_mailbox (void)
{
  GtkTreeSelection *selection;

  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(mn_preferences.list));
  gtk_tree_selection_selected_foreach(selection, mn_preferences_remove_mailbox_cb, NULL);
}

static void
mn_preferences_remove_mailbox_cb (GtkTreeModel *model,
				  GtkTreePath *path,
				  GtkTreeIter *iter,
				  gpointer data)
{
  MNMailbox *mailbox;

  gtk_tree_model_get(model, iter, COLUMN_PTR, &mailbox, -1);
  mn_conf_remove_mailbox(mailbox->locator);
}
