/* 
 * Copyright (c) 2003 Jean-Yves Lefort <jylefort@brutele.be>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "config.h"
#include <gnome.h>
#include "art/mail.h"
#include "art/no_mail.h"
#include "mn-conf.h"
#include "mn-dialog.h"
#include "eggtrayicon.h"
#include "glade-interface.h"

/*** cpp *********************************************************************/

#define MAIL_NOTIFICATION_STOCK_MAIL		"mail-notification-mail"
#define MAIL_NOTIFICATION_STOCK_NO_MAIL		"mail-notification-no-mail"

/*** variables ***************************************************************/

static GtkTooltips	*mn_ui_tooltips;
static EggTrayIcon	*mn_ui_tray_icon;
static GtkWidget	*mn_ui_image;
static GtkWidget	*mn_ui_menu;

/*** functions ***************************************************************/

static void	mn_ui_stock_init	(void);
static void	mn_ui_create_icon	(void);

static gboolean	mn_ui_icon_press	(GtkWidget	*widget,
					 GdkEventButton	*event,
					 gpointer	data);
static gboolean	mn_ui_icon_release	(GtkWidget	*widget,
					 GdkEventButton	*event,
					 gpointer	data);

/*** implementation **********************************************************/

static void
mn_ui_stock_init (void)
{
  const struct
  {
    const char	*stock_id;

    const char	*data;
    int		data_size;
  } icons[] = {
    {
      MAIL_NOTIFICATION_STOCK_MAIL,
      art_mail,
      sizeof(art_mail)
    },
    {
      MAIL_NOTIFICATION_STOCK_NO_MAIL,
      art_no_mail,
      sizeof(art_no_mail)
    }
  };
      
  GtkIconFactory *factory;
  int i;

  factory = gtk_icon_factory_new();
  gtk_icon_factory_add_default(factory);

  for (i = 0; i < G_N_ELEMENTS(icons); i++)
    {
      GtkIconSet *icon_set;
      GdkPixbuf *pixbuf;
      
      pixbuf = gdk_pixbuf_new_from_inline(icons[i].data_size,
					  icons[i].data,
					  FALSE,
					  NULL);
      g_assert(pixbuf != NULL);

      icon_set = gtk_icon_set_new_from_pixbuf(pixbuf);
      gtk_icon_factory_add(factory, icons[i].stock_id, icon_set);

      gtk_icon_set_unref(icon_set);
      g_object_unref(pixbuf);
    }
  
  g_object_unref(factory);
}

static void
mn_ui_create_icon (void)
{
  GtkWidget *event_box;
  
  /* create widgets */

  mn_ui_tooltips = gtk_tooltips_new();
  mn_ui_tray_icon = egg_tray_icon_new("Mail Notification");
  event_box = gtk_event_box_new();
  mn_ui_image = gtk_image_new_from_stock(MAIL_NOTIFICATION_STOCK_NO_MAIL, GTK_ICON_SIZE_MENU);

  gtk_tooltips_set_tip(mn_ui_tooltips,
		       GTK_WIDGET(mn_ui_tray_icon),
		       _("You have no new mail"),
		       NULL);

  /* pack widgets */

  gtk_container_add(GTK_CONTAINER(event_box), mn_ui_image);
  gtk_container_add(GTK_CONTAINER(mn_ui_tray_icon), event_box);

  /* bind signals */

  g_signal_connect(G_OBJECT(event_box), "button-press-event",
		   G_CALLBACK(mn_ui_icon_press), NULL);
  g_signal_connect(G_OBJECT(event_box), "button-release-event",
		   G_CALLBACK(mn_ui_icon_release), NULL);

  /* show widgets */

  gtk_widget_show(mn_ui_image);
  gtk_widget_show(event_box);
  gtk_widget_show(GTK_WIDGET(mn_ui_tray_icon));
}

static gboolean
mn_ui_icon_press (GtkWidget *widget,
		  GdkEventButton *event,
		  gpointer data)
{
  if (event->button == 1 && mn_conf_get_bool("/apps/mail-notification/commands/clicked/enabled"))
    {
      const char *command;

      command = mn_conf_get_string("/apps/mail-notification/commands/clicked/command");
      if (command)
	{
	  GError *err = NULL;
	  
	  if (! g_spawn_command_line_async(command, &err))
	    {
	      mn_error_dialog(_("Command error."),
			      _("Unable to execute clicked command: %s."),
			      err->message);
	      g_error_free(err);
	    }
	}
    }
  else if (event->button == 3)
    {
      gtk_menu_popup(GTK_MENU(mn_ui_menu), NULL, NULL, NULL, NULL,
		     event->button, event->time);
      return TRUE;
    }

  return FALSE;
}

static gboolean
mn_ui_icon_release (GtkWidget *widget,
		    GdkEventButton *event,
		    gpointer data)
{
  if (event->button == 3)
    {
      gtk_menu_popdown(GTK_MENU(mn_ui_menu));
      return FALSE;
    }
  
  return TRUE;
}

void
mn_ui_init (void)
{
  mn_ui_stock_init();
  mn_ui_menu = create_menu();
  mn_ui_create_icon();
}

void
mn_ui_has_new (gboolean has_new)
{
  gtk_image_set_from_stock(GTK_IMAGE(mn_ui_image),
			   has_new ? MAIL_NOTIFICATION_STOCK_MAIL : MAIL_NOTIFICATION_STOCK_NO_MAIL,
			   GTK_ICON_SIZE_MENU);
  gtk_tooltips_set_tip(mn_ui_tooltips,
		       GTK_WIDGET(mn_ui_tray_icon),
		       has_new ? _("You have new mail") : _("You have no new mail"),
		       NULL);
}
